/*
 * Copyright (c) 2010 Ant Kutschera, maxant
 * 
 * This file is part of Ant Kutschera's blog.
 * 
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with this software.  If not, see <http://www.gnu.org/licenses/>.
 */
package ch.maxant.traincompany.businesslayer.timetable;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;

import javax.ejb.Stateless;

import ch.maxant.traincompany.bom.Locomotive;
import ch.maxant.traincompany.bom.Segment;
import ch.maxant.traincompany.bom.Train;
import ch.maxant.traincompany.bom.Trip;
import ch.maxant.traincompany.bom.TripSegment;
import ch.maxant.traincompany.bom.Wagon;

/**
 * TimeTableService provides schedulted passenger train trips
 * as well as bespoke cargo trips.
 */
@Stateless
public class Timetable implements TimetableLocal {

	private static final Random RANDOM = new Random();
	
	/**
	 * @return the list of trips occurring during the given
	 * period.  in this stub service, it always returns a 
	 * hard coded list, and ignores the from and to parameters.
	 */
	@Override
	public List<Trip> getTripsForPeriod(Date from, Date to) {
		List<Trip> trips = new ArrayList<Trip>();
		
		for(int i = 0; i < 100 + RANDOM.nextInt(300); i++){
			trips.add(createTrip());
		}
		return trips;
	}

	private Trip createTrip() {
		Trip trip = new Trip();
		Train train = new Train();
		train.setDragCoefficient(0.01 + (RANDOM.nextDouble()/20.0));
		Wagon wagon = new Wagon();
		wagon.setRollingFriction(0.01 + (RANDOM.nextDouble()/20.0));
		wagon.setWeightKg((10 + RANDOM.nextInt(10)) * 1000);
		train.getWagons().add(wagon);
		Locomotive loc = new Locomotive();
		loc.setRollingFriction(0.01 + (RANDOM.nextDouble()/20.0));
		loc.setWeightKg((30 + RANDOM.nextInt(10)) * 1000);
		loc.setEfficiency(0.80 + (RANDOM.nextDouble()/1000.0));
		train.getLocomotives().add(loc );
		trip.setTrain(train );
		TripSegment tripSegment = new TripSegment();
		Segment segment = new Segment();
		segment.setAltitudeChange(RANDOM.nextInt(200)-100);
		segment.setLengthMeters((1 + RANDOM.nextInt(9)) * 1000); // 1-10 km
		tripSegment.setSegment(segment);
		tripSegment.setSpeed(30 + RANDOM.nextInt(30)); //30-60 kmh
		trip.getTripSegments().add(tripSegment);
		return trip;
	}
}
