/*
 * Copyright (c) 2010 Ant Kutschera, maxant
 * 
 * This file is part of Ant Kutschera's blog.
 * 
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with this software.  If not, see <http://www.gnu.org/licenses/>.
 */
package ch.maxant.traincompany.web;

import java.io.IOException;

import javax.annotation.Resource;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.annotation.WebServlet;

import ch.maxant.traincompany.businesslayer.forecastingcontext.EnergyRequirementResult;
import ch.maxant.traincompany.processlayer.EnergyForecastingServiceLocal;

import java.io.PrintWriter;
import java.text.DecimalFormat;

/**
 * Servlet implementation class EnergyForecaster
 */
@WebServlet(name = "EnergyForecaster", urlPatterns = { "/EnergyForecaster" })
public class EnergyForecaster extends HttpServlet {

	private static final long serialVersionUID = 1L;

	/** access to the process layer */
	@Resource(name="java:app/TrainCompanyEJB/EnergyForecastingService")
	private EnergyForecastingServiceLocal forcaster;

	/** {@inheritDoc} */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		processRequest(request, response);
	}

	/** {@inheritDoc} */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		processRequest(request, response);
	}

	/**
	 * Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
	 * @param request servlet request
	 * @param response servlet response
	 * @throws ServletException if a servlet-specific error occurs
	 * @throws IOException if an I/O error occurs
	 */
	protected void processRequest(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		response.setContentType("text/html;charset=UTF-8");
		PrintWriter out = response.getWriter();
		try {
			out.println("<html>");
			out.println("<head>");
			out.println("<title>Energy Forecast</title>");
			out.println("</head>");
			out.println("<body>");
			out.println("<h1>Energy Forecast</h1>");
			
			EnergyRequirementResult result = forcaster.forecast();
			
			DecimalFormat df = new DecimalFormat("#"); //no decimal places
			out.println("For " + result.getTrips().size() + " trips, we need " + df.format(result.getEnergyRequirement()/1000000.0) + " MJ. Calculated in " + (df.format(result.getCalculationTimeNs()/1000000.0)) + " ms.");
			
			out.println("</body>");
			out.println("</html>");
		} catch (Exception e) {
			throw new ServletException(e);
		} finally {
			out.close();
		}
	}

}
